#!/bin/bash
#
############ BUILD SCRIPT FOR WL1271 SUPPORT PACKAGES FOR ANDROID ##############
#                                                                              #
# File Name     : WL1271AndroidBuild.sh                                        #
#                                                                              #
# Date          : 13th January 2011                                            #
#                                                                              #
# Based on WL1271Build.sh supplied with WL1271_Linux_SDK 3_00_01_06            #
################################################################################

#          Rowboat Android path : 
ROWBOAT_ROOT_PATH=/home/<username>/rowboat

ARM_TOOLCHAIN_PATH=$ROWBOAT_ROOT_PATH/prebuilt/linux-x86/toolchain/arm-eabi-4.4.0

KERNEL_EXTRACTED_SOURCE_PATH=$ROWBOAT_ROOT_PATH/kernel

WL1271_BUILD_ROOT_PATH=$PWD

#          PSP VERSION : Value ( 2 or 3) - Required for proper error checking
#
#          PSP Versions:
#                     1.  PSP-02.01.03.11 (linux-2.6.29)
#                     2.  PSP-03.00.01.06 (linux-2.6.32)
#
#          Usually PSP_REL_VER is the version of linux tar ball
PSP_MAIN_VER=3
PSP_REL_VER=03.00.01.06

echo "WL1271_BUILD_ROOT_PATH            : $WL1271_BUILD_ROOT_PATH"
echo "PSP_MAIN_VER                      : $PSP_MAIN_VER"
echo "PSP_REL_VER                       : $PSP_REL_VER"
echo "ARM_TOOLCHAIN_PATH                : $ARM_TOOLCHAIN_PATH"
echo "KERNEL_EXTRACTED_SOURCE_PATH      : $KERNEL_EXTRACTED_SOURCE_PATH"
echo ""
################################################################################

# MACRO Define Section

# Don't edit the following sections

# 1 - Common 
SLEEP_INTERVAL=1
NO_MODULE=""
KERNEL_MODULE="kernel"
WLAN_MODULE="wlan"
COMMON_PATH=$PWD
WL1271_BUILD_ROOT_PATH=$PWD
IMAGE_PATH=$COMMON_PATH/image
IMPORT_PATH=$COMMON_PATH/common
LOG_PATH=$COMMON_PATH/buildlog
DEMO=$IMAGE_PATH/WL1271_demo_01
COMMON_PATCH_PATH=$COMMON_PATH/common/patches

# 2 - WLAN
WLAN_PATH=$COMMON_PATH/wireless
WLAN_BUILD_PATH=$COMMON_PATH/wireless/WiLink
CUDK_PATH=$WLAN_PATH/WiLink/CUDK
SUPPLICANT_PATH=$WLAN_PATH/WiLink/CUDK/wpa_suppl


# Edit the following MACROS if PSP version or WiLink version get changed
WILINK_REL_VER=WiLink6.1.6.0
WL1271_REL_VER=WL6.1.6.0

export DEMO_ROOT_DIR=$BT_PATH

check_build_setup () {

	echo ""
	echo "Env variables are taken from the file 'set_wl1271_build_env.sh'"
	echo ""
	sleep $SLEEP_INTERVAL

	echo ""
	echo "Checking build setup."
	echo "It may take few seconds, Please wait ..."
	echo ""

	sleep $SLEEP_INTERVAL
 
	if [ ! -f "$ARM_TOOLCHAIN_PATH"/bin/arm-eabi-gcc ]; then
		echo "Compiler "$ARM_TOOLCHAIN_PATH"/bin/
		arm-eabi-gcc does not exist!"
		exit
	else
		echo "Compiler "$ARM_TOOLCHAIN_PATH"/bin/
		arm-eabi-gcc Found!"
	fi

	sleep $SLEEP_INTERVAL

	if [ -d "$KERNEL_EXTRACTED_SOURCE_PATH" ] 
    	then
    		KERNEL_BUILD_PATH=$KERNEL_EXTRACTED_SOURCE_PATH
	else
    		echo "Kernel sources not found."
		echo "Verify that the updated kernel source is present at $KERNEL_EXTRACTED_SOURCE_PATH"
		exit
	fi

	echo "KERNEL_BUILD_PATH = "$KERNEL_BUILD_PATH""

        # export compiler path        
        export PATH=$ARM_TOOLCHAIN_PATH/bin:$PATH

}


# This function take the configuration file from the "common" folder and
# build the kernel. The "uImage" will be install under the "image" folder 
build_kernel_module() {
    echo ""
    echo "Kernel building started ..."
    echo "It may take few minutes, Please wait ..."
    echo ""

    ls $KERNEL_BUILD_PATH &> /dev/null
    if [ "$?"  -ne 0 ]; then
        echo "Kernel sources not found."
        echo "Verify that the kernel sources are available at $KERNEL_BUILD_PATH."
        exit
    fi

    ls $KERNEL_BUILD_PATH/.config &> /dev/null
    if [ "$?"  -ne 0 ]; then
       make CROSS_COMPILE=arm-eabi- ARCH=arm omap3_evm_defconfig
    fi


    ls $KERNEL_BUILD_PATH/.config &> /dev/null
    if [ "$?"  -ne 0 ]; then
        echo "There is no  default omap3_evm_config present ..."
        exit
    else
       cd $KERNEL_BUILD_PATH
    fi

# Do menuconfig
#   make CROSS_COMPILE=arm-eabi- ARCH=arm menuconfig
  
    ls $LOG_PATH &> /dev/null
    if [ "$?"  -ne 0 ]; then
        mkdir -p $LOG_PATH
    fi

    ls $LOG_PATH/kernelBuild.log &> /dev/null
    if [ "$?"  -ne 0 ]; then
        touch $LOG_PATH/kernelBuild.log
    fi

    tail -f $LOG_PATH/kernelBuild.log &
    make CROSS_COMPILE=arm-eabi- ARCH=arm uImage &> $LOG_PATH/kernelBuild.log
    if [ "$?"  -ne 0 ]; then
        echo "ERROR: Kernel Build failed ..."
        echo "Refer log file: $LOG_PATH/kernelBuild.log"
        killall -9 tail &> /dev/null
        exit
    fi
    killall -9 tail &> /dev/null

    ls $IMAGE_PATH &> /dev/null
    if [ "$?"  -ne 0 ]; then
        echo "Create the folder $IMAGE_PATH"
        mkdir -p $IMAGE_PATH
    fi

    cp $KERNEL_BUILD_PATH/arch/arm/boot/uImage $IMAGE_PATH/
    chmod a+wx $IMAGE_PATH/uImage
    echo "Kernel build Completed ..."
    echo ""
}


# This function will clean the kernel build.
clean_kernel_module() {
    echo ""
    echo "Kernel cleaning started ..."
    echo ""
 
    ls $KERNEL_BUILD_PATH &> /dev/null
    if [ "$?"  -ne 0 ]; then
        echo "There is no kernel module available to clean ..."
        exit 1
    fi
 
    cd $KERNEL_BUILD_PATH

    make CROSS_COMPILE=arm-eabi- ARCH=arm clean &> $LOG_PATH/kernelBuild.log
    if [ "$?"  -ne 0 ]; then
        echo "ERROR: Kernel Build failed ..."
        echo "Refer log file: $LOG_PATH/kernelBuild.log"
        exit 1
    fi

    echo "Kernel cleaning completed ..."
}

# This function will delete the kernel build.
del_kernel_module() {
    echo ""
    echo "Deleting "$KERNEL_BUILD_PATH" ..."
    echo ""
    sleep 1

    rm -rf $KERNEL_BUILD_PATH &> /dev/null

}


# This function build the wireless module. It can extract module from "common" folder
# and apply the patch; and compile the WLAN module. The compiled binary will be 
# available under the "image" folder 
build_wlan_module() {
    echo ""
    echo "Wireless module building ..."
  
    export PATH=$ARM_TOOLCHAIN_PATH/bin:$PATH 
    export CROSS_COMPILE=arm-eabi-
    export ARCH=arm
    export HOST_PLATFORM=omap3530
    export KERNEL_DIR=$KERNEL_BUILD_PATH
    export XCC=n 

    ls $WLAN_BUILD_PATH &> /dev/null
    if [ "$?" -ne 0 ]; then

        ls $IMPORT_PATH/wireless_src/$WILINK_REL_VER.tar.bz2 2> /dev/null
        if [ "$?" -ne 0 ]; then
            echo ""
            echo "There is no wireless module under \"$WLAN_PATH\""
            echo "There is no \""$WILINK_REL_VER"\" file present under \"$IMPORT_PATH/wireless_src\""
        fi

        echo "Extracting Wireless module sources ..."
        ls $WLAN_PATH 2> /dev/null
        if [ "$?" -ne 0 ]; then
            mkdir -p $WLAN_PATH
        fi

        cd $WLAN_PATH
        tar -xjf $IMPORT_PATH/wireless_src/$WILINK_REL_VER.tar.bz2 2> /dev/null

	if [ ! -d "$WLAN_BUILD_PATH" ]; then

		echo ""$WLAN_BUILD_PATH" sources does not exist "
		exit
	fi

        ls $IMPORT_PATH/wireless_src/wpa_supplicant-0.5.7.tar.gz 2> /dev/null
        if [ "$?" -ne 0 ]; then
            echo ""
            echo "There is no \"wpa_supplicant-0.5.7.tar.gz\" file present under \"$IMPORT_PATH/wireless_src\""
            echo ""
            echo -n "Do you need to retive file form Link(y/n)"
            read RET_LINK
            if [ $RET_LINK = "y" ]; then
                cd $IMPORT_PATH/wireless_src
                echo ""
                wget http://hostap.epitest.fi/releases/wpa_supplicant-0.5.7.tar.gz
                echo ""
            else
                echo "Manually copy the \"wpa_supplicant-0.5.7.tar.gz\" file present under \"$IMPORT_PATH/wireless_src\""
                exit
            fi
        fi
 
        cd $CUDK_PATH
        echo "Extracting WPA Supplicant ..."
        tar -xzf $IMPORT_PATH/wireless_src/wpa_supplicant-0.5.7.tar.gz
        cp -rf $CUDK_PATH/wpa_supplicant-0.5.7/* $SUPPLICANT_PATH/
        if [ "$?" -ne 0 ]; then
             echo "No Supplicant available ..."
        fi
        rm -rf $CUDK_PATH/wpa_supplicant-0.5.7
           
        ls $SUPPLICANT_PATH/wpa_suppl.diff &> /dev/null
        if [ "$?" -eq 0 ]; then
            cd $SUPPLICANT_PATH
            patch -p2 < wpa_suppl.diff
        fi

        ls $IMPORT_PATH/wireless_src/openssl-0.9.8e.tar.gz &> /dev/null
        if [ "$?" -ne 0 ]; then
            echo ""
            echo "There is no \"openssl-0.9.8e.tar.gz\" file present under \"$IMPORT_PATH/wireless_src\""
            echo ""
            echo -n "Do you need to retive file form Link(y/n)"
            read OPEN_SSL
            if [ $OPEN_SSL = "y" ]; then
                cd $IMPORT_PATH/wireless_src
                echo ""
                wget http://www.openssl.org/source/openssl-0.9.8e.tar.gz
                echo ""
            else
                echo "Manually copy the \"openssl-0.9.8e.tar.gz\" file present under \"$IMPORT_PATH/wireless_src\""
                exit
            fi
        fi
      
        cd $CUDK_PATH
        tar -xzf $IMPORT_PATH/wireless_src/openssl-0.9.8e.tar.gz
        cp $SUPPLICANT_PATH/openssl-0.9.8e/openssl-gcc42.diff $CUDK_PATH/openssl-0.9.8e/
        cd $CUDK_PATH/openssl-0.9.8e/
        patch -p1 < openssl-gcc42.diff
        cp $SUPPLICANT_PATH/openssl-0.9.8e/Makefile $CUDK_PATH/openssl-0.9.8e/
        
        cd $CUDK_PATH/openssl-0.9.8e/
        # Skipping openssl build, as not required
        # This is already done as part of main Android build
        #make clean
	#make
        if [ "$?" -ne 0 ]; then
            echo "Open SSL build Fail"
            exit
        fi

        cp -rf $CUDK_PATH/openssl-0.9.8e/* $SUPPLICANT_PATH/openssl-0.9.8e/
        if [ "$?" -ne 0 ]; then
             echo "No Open SSL file available ..."
             exit
        fi

        rm -rf $CUDK_PATH/openssl-0.9.8e 

        # Apply bug-fixes and android build workarounds
        cd $COMMON_PATH
        ls $COMMON_PATH/wl1271-android.patch &> /dev/null
        if [ "$?"  -ne 0 ]; then
            echo "Wl1271 patch not found!"
            echo "Check that the file \'wl1271-android.patch\' is present at $COMMON_PATH"
            exit
        fi

        echo "Applying patches for WL1271 Android build ..."
        patch -p1 < $COMMON_PATH/wl1271-android.patch
    fi

    echo "export PATH=$ARM_TOOLCHAIN_PATH/bin:\$PATH" > $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
    echo "export CROSS_COMPILE=arm-eabi-" >> $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
    echo "export ARCH=arm" >> $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
    echo "export HOST_PLATFORM=omap3530" >> $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
    echo "export KERNEL_DIR=$KERNEL_BUILD_PATH" >> $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
    echo "export XCC=n" >> $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
    echo "export TI_HOST_OS=ANDROID" >> $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
    echo "export SUPPL=ANDROID" >> $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
    
    chmod 777 $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
    cd $WLAN_BUILD_PATH/platforms/os/linux &> /dev/null

    source omap3530_env_noXCC.sh
    echo "Wireless module compilation started, Please Wait ..."
   
    ls $LOG_PATH &> /dev/null
    if [ "$?"  -ne 0 ]; then
        mkdir -p $LOG_PATH
    fi  
 
    ls $LOG_PATH/wlanBuild.log &> /dev/null
    if [ "$?"  -ne 0 ]; then
        touch $LOG_PATH/wlanBuild.log
    fi

    tail -f $LOG_PATH/wlanBuild.log &    

    make &> $LOG_PATH/wlanBuild.log 

    if [ "$?" -ne 0 ]; then
        echo ""
        echo "ERROR: Wireless Build failed ..."
        echo "Refer log file: $LOG_PATH/wlanBuild.log"
        echo ""
        echo "Use below command to build|rebuild individually wireless module ....."
        echo ""
        echo "Usage:./WL1271Build.sh build|rebuild wlan"
        killall -9 tail   
        exit
    fi

    killall -9 tail &> /dev/null 

    ls $WLAN_BUILD_PATH/platforms/os/linux/omap3530Binaries.tar 2> /dev/null
    if [ "$?" -ne 0 ]; then
        echo "There is no tar ball wireless binary file (omap3530Binaries.tar)."
        exit
    fi
 
    ls $DEMO/wlan &> /dev/null
    if [ "$?" -ne 0 ]; then
        echo "Create wlan folder ..."
        mkdir -p $DEMO/wlan
    fi
    
    cp $WLAN_BUILD_PATH/platforms/os/linux/omap3530Binaries.tar $DEMO/wlan/
    
    ls $DEMO/wlan/omap3530Binaries.tar
    if [ "$?" -ne 0 ]; then
        echo "There is no \"omap3530Binaries.tar\" file available"
    else
        cp $WLAN_BUILD_PATH/platforms/os/linux/omap3530Binaries.tar $DEMO/wlan/
        cd $DEMO/wlan/
        tar -xf omap3530Binaries.tar
        rm -rf $DEMO/wlan/omap3530Binaries.tar
    fi

    echo ""
    echo "Wireless module build Completed ..."
    echo ""
}


# This function will clean the wireless module build
clean_wlan_module() {
    echo ""
    echo "Wireless module cleaning ..."

    ls $WLAN_BUILD_PATH &> /dev/null
    if [ "$?" -ne 0 ]; then
        echo "ERROR: Wireless Clean Failed ..."
        echo "There is no Wireless Module Available."
        return
    fi

    cd $WLAN_BUILD_PATH/platforms/os/linux

    ls  $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh &> /dev/null
    if [ "$?" -ne 0 ]; then
        echo "export PATH=$ARM_TOOLCHAIN_PATH/bin:\$PATH" > $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
        echo "export CROSS_COMPILE=arm-eabi-" >> $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
        echo "export ARCH=arm" >> $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
        echo "export HOST_PLATFORM=omap3530" >> $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
        echo "export KERNEL_DIR=$KERNEL_BUILD_PATH" >> $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
        echo "export XCC=n" >> $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
        echo "export TI_HOST_OS=ANDROID" >> $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
        echo "export SUPPL=ANDROID" >> $WLAN_BUILD_PATH/platforms/os/linux/omap3530_env_noXCC.sh
    fi

    cd $WLAN_BUILD_PATH/platforms/os/linux
    source omap3530_env_noXCC.sh

    cd $WLAN_BUILD_PATH/platforms/os/linux
    make clean &> $LOG_PATH/wlanBuild.log

    echo "Wireless Module Cleaning Completed ..."
}

# This function will delete the wlan build.
del_wlan_module() {
    echo ""
    echo "Deleting "$WLAN_BUILD_PATH" ..."
    echo ""
    sleep 1

    rm -rf $WLAN_BUILD_PATH &> /dev/null

}



# This case will take the parameter for build|rebuild|buildclean
# for the modules kernel|wlan|all. This script is required
# atleast 3 parameters for execution.
case "$1" in
    build)
    if [ $# -lt 2 ]; then
        echo ""
        echo "Follow the below help to build the modules .........."
        echo ""
        $0 --help
        exit
    fi
    echo ""
    echo "Starting WL1271 source package build"
    echo ""
    sleep $SLEEP_INTERVAL

    check_build_setup

    echo "Building the modules ..."
    if [ "$2" = $KERNEL_MODULE -o "$3" = $KERNEL_MODULE ]; then
        build_kernel_module
    fi

    if [ "$2" = $WLAN_MODULE -o "$3" = $WLAN_MODULE ]; then
        build_wlan_module
    fi

    if [ "$2" = "all" ]; then
        build_kernel_module
        build_wlan_module
    fi

    echo "Build modules Completed..."
    echo ""
    echo "All compiled modules binary available under \"$IMAGE_PATH\" folder"
    echo ""
    ;;
    buildclean)
    if [ $# -lt 2 ]; then
        echo ""
        echo "Follow the below help to build the modules .........."
        echo ""
        $0 --help
        exit
    fi
    
    check_build_setup

    echo ""
    echo "Cleaning the modules ..."
    if [ "$2" = $KERNEL_MODULE -o "$3" = $KERNEL_MODULE ]; then
        clean_kernel_module
    fi

    if [ "$2" = $WLAN_MODULE -o "$3" = $WLAN_MODULE ]; then
        clean_wlan_module
    fi

    if [ "$2" = "all" ]; then
        clean_kernel_module
        clean_wlan_module
    fi
    echo ""
    echo "Clean modules Completed..."
    echo ""
    ;;
    rebuild)
    if [ $# -lt 2 ]; then
        echo ""
        echo "Follow the below help to build the modules .........."
        echo ""
        $0 --help
        exit
    fi
    echo ""
    echo "Rebuilding the modules ..." 
    echo ""
    $0 buildclean $2 $3
    sleep "$SLEEP_INTERVAL"
    $0 build $2 $3
    echo "Rebuild Completed ..."
    echo ""
    ;;
    --help)
    echo ""
    echo "Collect Information to building the modules"
    echo "Usage:./WL1271Build.sh [OPTION] [MODULE NAME 1] [MODULE NAME 2]"
    echo ""
    echo "[OPTION]:"
    echo ""
    echo "       build       --Build modules"
    echo "       rebuild     --Rebuild modules"
    echo "       buildclean  --Clean the build modules"
    echo "       builddel    --Delete build folders"
    echo ""
    echo "[MODULE NAME]:"
    echo "" 
    echo "       kernel      --Kernel build" 
    echo "       wlan        --Wlan module"
    echo "       all         --Build All modules" 
    echo "" 
    ;;

    builddel)
    if [ $# -lt 2 ]; then
        echo ""
        echo "Follow the below help to build the modules .........."
        echo ""
        $0 --help
        exit
    fi
 
    check_build_setup

    echo ""
    echo "Deleting the modules ..."
    if [ "$2" = $KERNEL_MODULE -o "$3" = $KERNEL_MODULE ]; then
        del_kernel_module
    fi

    if [ "$2" = $WLAN_MODULE -o "$3" = $WLAN_MODULE ]; then
        del_wlan_module
    fi

    if [ "$2" = "all" ]; then
        del_kernel_module
        del_wlan_module
    fi
    echo ""
    echo "Deleting build folders Completed..."
    echo ""
    ;;

    *)
    echo ""
    echo "Invalid command ..."
    echo "USE: --help to choose the option ..."
    echo ""
    ;;
esac



